#!/usr/bin/env python

"""Python wrapper for the AIDSinfo drug information API."""

from api import API

class DrugInfo(API):
    """Python wrapper for the AIDSinfo drug information API."""

    def __init__(self):
        super(DrugInfo, self).__init__()
        self.base_url = 'http://aidsinfo.nih.gov/DrugsNew/drug.aspx'
        self.output_format = 'xml'
        self.required_params = {'displayxml': 'true'}

    def search(self, name, **kwargs):
        """Search for a specific drug's information."""
        kwargs.update({'name': name})
        self.call_api(**kwargs)



"""
object_dict

nkchenz@gmail.com 2007

Provided as-is; use at your own risk; no warranty; no promises; enjoy!
"""

class object_dict(dict):
    """Object view of dict, you can.
    >>> a = object_dict()
    >>> a.fish = 'fish'
    >>> a['fish']
    'fish'
    >>> a['water'] = 'water'
    >>> a.water
    'water'
    >>> a.test = {'value': 1}
    >>> a.test2 = object_dict({'name': 'test2', 'value': 2})
    >>> a.test, a.test2.name, a.test2.value
    (1, 'test2', 2)
    """

    def __init__(self, initd=None):
        if initd is None:
            initd = {}
        dict.__init__(self, initd)

    def __getattr__(self, item):
        d = self.__getitem__(item)
        # if value is the only key in object, you can omit it
        if isinstance(d, dict) and 'value' in d and len(d) == 1:
            return d['value']
        else:
            return d

    def __setattr__(self, item, value):
        self.__setitem__(item, value)



#!/usr/bin/env python

"""Thunder Chen<nkchenz@gmail.com> 2007.9.1"""

from __future__ import with_statement
import re

try:
    import xml.etree.ElementTree as ET
except:  # pragma: no cover
    # For Python 2.4
    import cElementTree as ET

from object_dict import object_dict


class XML2Dict(object):

    def _parse_node(self, node):
        node_tree = object_dict()
        if node.text and node.attrib:
            if node.tag in node.attrib:
                raise ValueError("Name conflict: Attribute name conflicts with "
                                 "tag name. Check the documentation.")
            node.attrib.update({node.tag: node.text})
            node.text = ''
        # Save attrs and text. Fair warning, if there's a child node with the same name
        # as an attribute, values will become a list.
        if node.text and node.text.strip():
            node_tree = node.text
        else:
            for k, v in node.attrib.items():
                k, v = self._namespace_split(k, v)
                node_tree[k] = v
            # Save children.
            for child in node.getchildren():
                tag, tree = self._namespace_split(child.tag, self._parse_node(child))
                if tag not in node_tree:  # First encounter, store it in dict.
                    node_tree[tag] = tree
                    continue
                old = node_tree[tag]
                if not isinstance(old, list):
                    # Multiple encounters, change dict to a list
                    node_tree.pop(tag)
                    node_tree[tag] = [old]
                node_tree[tag].append(tree)  # Add the new one.
        if not node_tree:
            node_tree = None
        return node_tree

    def _namespace_split(self, tag, value):
        """
           Split the tag  '{http://cs.sfsu.edu/csc867/myscheduler}patients'
             ns = http://cs.sfsu.edu/csc867/myscheduler
             name = patients
        """
        result = re.compile("\{(.*)\}(.*)").search(tag)
        if result:
            tag = result.groups(1)
            # value.namespace, tag = result.groups()
        return (tag, value)

    def parse(self, file):
        """Parse an XML file to a dict."""
        with open(file, 'r') as f:
            return self.fromstring(f.read())

    def fromstring(self, s):
        """Parse a string."""
        t = ET.fromstring(s)
        root_tag, root_tree = self._namespace_split(t.tag, self._parse_node(t))
        return object_dict({root_tag: root_tree})


class Dict2XML(object):
    """Turn a dictionary into an XML string."""

    def tostring(self, d):
        """Convert dictionary to an XML string."""
        if not isinstance(d, dict):
            raise TypeError('tostring must receive a dictionary: %r' % d)
        if len(d) != 1:
            raise ValueError('Dictionary must have exactly one root element')
        if isinstance(d.itervalues().next(), list):
            raise ValueError('Dictionary must not be a map to list: %r' % d)

        xml_list = ['<?xml version="1.0" encoding="UTF-8" ?>\n']
        xml_list.append(self.__tostring_helper(d))
        return ''.join(xml_list)

    def __tostring_helper(self, d):
        if isinstance(d, int):
            return str(d)

        elif isinstance(d, basestring):
            return '<![CDATA[%s]]>' % d

        elif isinstance(d, dict):
            x = []
            for tag, content in d.iteritems():
                if content is None:
                    x.append('<%s />' % tag)
                elif isinstance(content, list):
                    for c in content:
                        if c is None:
                            x.append('<%s />' % tag)
                        else:
                            x.append('<%s>%s</%s>' %\
                                     (tag, self.__tostring_helper(c), tag))
                else:
                    x.append('<%s>%s</%s>' %\
                             (tag, self.__tostring_helper(content), tag))
            xml_string = ''.join(x)
            return xml_string

        else:
            raise ValueError('Cannot convert %r to an XML string' % d)



#!/usr/bin/env python

"""Python wrapper for an API."""

try:
    import json
except ImportError:  # pragma: no cover
    # For older versions of Python.
    import simplejson as json

try:
    from urllib import urlencode
except ImportError:  # pragma: no cover
    # For Python 3.
    from urllib.parse import urlencode

try:
    from urllib2 import urlopen
except ImportError:  # pragma: no cover
    # For Python 3.
    from urllib.request import urlopen

from xml2dict import xml2dict


class API(object):
    """An example class for a Python API wrapper."""

    def __init__(self, api_key=''):
        if api_key:
            self.api_key = api_key
        self.base_url = ''
        self.output_format = None
        self.required_params = None

    def call_api(self, directory=None, **kwargs):
        """
        A generic example api wrapping method. Other methods can use this
        method to interact with the API.
        """
        self._check_base_url()
        url_list = [self.base_url]
        if directory:
            url_list.append('/%s' % directory)
        if self.required_params:
            kwargs.update(self.required_params)
        try:
            output_format = kwargs.pop('output_format')
        except KeyError:
            output_format = self.output_format
        if kwargs:
            params = urlencode(kwargs)
            url_list.extend(['?', params])
        url = ''.join(url_list)
        data = urlopen(url).read()
        return self._format_data(output_format, data)

    def _check_base_url(self):
        """Internal method to format `self.base_url`."""
        base_url = self.base_url
        if base_url and base_url.endswith('/'):
            base_url = base_url.rstrip('/')
            self.base_url = base_url

    def _format_data(self, output_format, data):
        """Internal method to return formatted data to developer."""
        if output_format:
            # Check for cases people capitalize JSON or XML.
            output_format =  output_format.lower()
        if output_format == 'json':
            # Turn JSON into a dictionary.
            return json.loads(data)
        elif output_format == 'xml':
            return self._xml_to_dict(data)
        return data

    def _xml_to_dict(self, xml):
        """
        Internal method to turn XML to dictionary output. Developers can
        overwrite this method to use their favorite XML parser of choice.
        """
        return xml2dict(xml)



#!/usr/bin/env python

import unittest

from mock import Mock

import api
from api import API


class TestApiClass(unittest.TestCase):

    def test_empty_api_initialization(self):
        my_api = API()
        self.assertEquals(my_api.base_url, '')
        self.assertEquals(my_api.output_format, None)
        self.assertEquals(my_api.required_params, None)

    def test_api_init_with_new_api_key(self):
        my_api = API('new_api_key')
        self.assertEquals(my_api.api_key, 'new_api_key')


class TestCallApiMethod(unittest.TestCase):

    def setUp(self):
        api.urlopen = Mock()

    def test_call_api_method(self):
        API('fake_key').call_api('example')
        expected_call = '/example'
        api.urlopen.assert_called_with(expected_call)

    def test_call_api_method_with_kwargs(self):
        API('fake_key').call_api('example', foo='bar')
        expected_call = '/example?foo=bar'
        api.urlopen.assert_called_with(expected_call)

    def test_call_api_method_without_output_formatting(self):
        API('fake_key').call_api('example', foo='bar', output_format=None)
        expected_call = '/example?foo=bar'
        api.urlopen.assert_called_with(expected_call)

    def test_call_api_method_with_required_params(self):
        my_api = API('fake_key')
        my_api.required_params = {'api_key': my_api.api_key}
        my_api.call_api('example', foo='bar')
        expected_call = '/example?api_key=fake_key&foo=bar'
        api.urlopen.assert_called_with(expected_call)


class TestCheckBaseUrlMethod(unittest.TestCase):

    def test_url_without_trailing_slash(self):
        api = API()
        api.base_url = 'http://mysite.com'
        api._check_base_url()
        self.assertEquals(api.base_url, 'http://mysite.com')

    def test_url_with_trailing_slash(self):
        api = API()
        api.base_url = 'http://mysite.com/'
        api._check_base_url()
        self.assertEquals(api.base_url, 'http://mysite.com')


class TestFormatDataMethod(unittest.TestCase):

    def test_json_output(self):
        data = '{"a": {"b": "c"}}'
        output = API()._format_data('json', data)
        expected_output = {'a': {'b': 'c'}}
        self.assertEquals(output, expected_output)

    def test_xml_output(self):
        data = ('<?xml version="1.0" encoding="utf-8" ?>'
                '<a><b>c</b></a>')
        output = API()._format_data('xml', data)
        expected_output = {'a': {'b': 'c'}}
        self.assertEquals(output, expected_output)

    def test_without_formatting_output(self):
        data = "ohai, I'm data"
        output = API()._format_data(None, data)
        self.assertEquals(output, data)


class ExampleJson(API):

    def __init__(self, api_key=''):
        super(ExampleJson, self).__init__(api_key)
        self.base_url = 'http://example.com'
        self.output_format = 'json'
        self.api_key = 'json_api_key'


class ExampleXml(API):

    def __init__(self, api_key=''):
        super(ExampleXml, self).__init__(api_key)
        self.base_url = 'http://example.com'
        self.output_format = 'xml'
        self.api_key = 'xml_api_key'


class ExampleKeywords(API):

    def __init__(self):
        super(ExampleKeywords, self).__init__()
        self.base_url = 'http://example.com'
        self.output_format = 'json'
        self.required_params = {'foo': 'bar'}


class TestExamples(unittest.TestCase):

    def setUp(self):
        api.urlopen = Mock()
        api.json = Mock()

    def tearDown(self):
        import json
        api.json = json

    def test_example_json_api(self):
        example = ExampleJson()
        self.assertEquals(example.api_key, 'json_api_key')
        self.assertEquals(example.base_url, 'http://example.com')
        self.assertEquals(example.output_format, 'json')

    def test_example_xml_api(self):
        example = ExampleXml()
        self.assertEquals(example.api_key, 'xml_api_key')
        self.assertEquals(example.base_url, 'http://example.com')
        self.assertEquals(example.output_format, 'xml')

    def test_example_with_new_required_params(self):
        example = ExampleKeywords()
        example.call_api('test')
        expected_url = 'http://example.com/test?foo=bar'
        api.urlopen.assert_called_with(expected_url)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Author: Zach Williams, <zach AT codeforamerica DOT org>

Copyright (c) 2011, Code for America. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. Redistributions in binary form
must reproduce the above copyright notice, this list of conditions and the
following disclaimer in the documentation and/or other materials provided with
the distribution. Neither the name of Code for America nor the names of its
contributors may be used to endorse or promote products derived from this
software without specific prior written permission. THIS SOFTWARE IS PROVIDED
BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


long_description = """
AIDSinfo Python API Wrapper
===========================

A Python wrapper for the AIDSinfo drug information API.

AIDSinfo Documentation:  http://www.aidsinfo.nih.gov/Other/rss.aspx


Usage
-----

    >>> from aidsinfo import DrugInfo
    >>> info = DrugInfo()
    >>> info.search('abacavir')
    {'abacavir': {'data': 'here'}}

    >>> info.search('combivir')
    {'combivir': {'data': 'here'}}

    >>> # You can also get back just the XML data.
    ... xml_data = info.search('combivir', output_format=None)


Copyright
---------

Copyright (c) 2011 Code for America Laboratories.

See LICENSE for details.
"""

setup(name="aidsinfo",
      version="1.1",
      description="A Python wrapper for the AIDSinfo drug information API.",
      long_description=long_description,
      keywords="aids, aids info, AIDSinfo, aidsinfo",
      author="Zach Williams",
      author_email="zach@codeforamerica.org",
      url="https://github.com/zachwill/fred",
      license="BSD",
      packages=['aidsinfo', 'aidsinfo.api', 'aidsinfo.api.xml2dict'],
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Intended Audience :: Developers',
                   'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python :: 2',
                   'Topic :: Internet',
                   'Topic :: Internet :: WWW/HTTP',
                  ],
      test_suite="test.py",
      tests_require=["mock", "Mock"])



"""
xml2dict
Thunder Chen<nkchenz@gmail.com> 2007.9.1

Convert an XML string or file with XML data to a dict object.
http://code.google.com/p/xml2dict/
"""

from .xml2dict import XML2Dict, Dict2XML
from object_dict import object_dict


def xml2dict(data):
    """Turn XML into a dictionary."""
    converter = XML2Dict()
    if hasattr(data, 'read'):
        # Then it's a file.
        data = data.read()
    return converter.fromstring(data)


def dict2xml(data):
    """Turn a dictionary into XML."""
    converter = Dict2XML()
    return converter.tostring(data)


__all__ = [XML2Dict, Dict2XML, xml2dict, dict2xml, object_dict]
